//*******************************************************
//
//	FM Radio in BCL Radio Figure
//
//	Tuner Chip : AR1000/AR1010
//
//	MCU : ATtiny13
//		hfuse:FBh	BOD 2.7V
//		lfuse:22h	0.6MHz
//
//	Compiler:	AVR-GCC		4.3.5
//				AVR-LIBC	1.7.1
//				binutils	2.20.1
//
//	2012.5.25	KAWAKAMI Yukio
//*******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>
#include	<avr/eeprom.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"main.h"
#include	"AR1000.h"

#define		R3_VOLUME	R3_VOLUMN_12		// AR1000 出力ボリューム(最大)

#define		SEEK_THRESHOLD	20			// Seek スレッショルド
#define		BAND_WIDTH	0x1000			// 0x1000:76-90MHz	0x1800 76-108MHz

// AR1000 レジスタ初期値
PROGMEM ushort	DefaultRegs[] = {
	R0_DEFAULT | R0_XO_EN,
	R1_DEFAULT | R1_HMUTE | R1_MONO,
	R2_DEFAULT,
	R3_DEFAULT,
	0x0400,
	0x28AA,
	0x4400,
	0x1EE7,
	0x7141,
	0x007D,
	0x82CE,
	0x4F55,
	0x970C,
	0xB845,
	0xFC2D,
	0x8097,
	0x04A1,
	0xDF6A
};

void write_AR1000(uchar adrs, ushort data);
ushort read_AR1000(const uchar adrs);

extern volatile uchar	Timer;


//*******************************************************
//	Init FM Chip
//*******************************************************
void init_fm_chip(void){
	ushort	reg;

	// AR1000 レジスタ初期値
	for (reg = 0; reg <= 17; reg++){
		write_AR1000(reg, pgm_read_word(&DefaultRegs[reg]));
	}
	write_AR1000(R0, R0_DEFAULT | R0_XO_EN | R0_ENABLE);	// ここで ENABLE

	wait_stc();
}


//*******************************************************
//	Seek UP
//*******************************************************
ushort seek_up(void){
	ushort	freq;

	// Seek開始
	write_AR1000(R3, BAND_WIDTH | R3_VOLUME | R3_SPACE | R3_SEEKUP | SEEK_THRESHOLD);
	write_AR1000(R3, BAND_WIDTH | R3_VOLUME | R3_SPACE | R3_SEEKUP | SEEK_THRESHOLD | R3_SEEK);

	wait_stc();

	freq = (read_AR1000(STATUS) >> 7) + 690;

	set_freq(freq);

	return freq;
}


//*******************************************************
//	STCフラグ待ち(Timerを使って LED点滅)
//*******************************************************
void wait_stc(void){

	for (;;){
		wait_timer(1);	// 10ms WAIT
		if (read_AR1000(STATUS) & STATUS_STC)	break;	// STC=H 待ち
		if (!Timer){
			Timer = 20;
			PINB |= (1<<LED);	// LED点滅
		}
	}

	PORTB |= (1<<LED);	// LED点灯
}


//*******************************************************
//	周波数設定
//*******************************************************
void set_freq(ushort freq){
	ushort	r2;

	r2 = freq - 690;
	write_AR1000(R2, r2);	// 周波数設定
	r2 |= R2_TUNE;
	write_AR1000(R2, r2);	// TUNE = ON
	wait_stc();
	write_AR1000(R1, R1_DEFAULT | R1_MONO);	// ミュート解除
}


//*******************************************************
//	AR1000 レジスタ書き込み(TWI)
//*******************************************************
void write_AR1000(uchar adrs, ushort data){

	TWI_start();
	TWI_send(0x20);			// スレーブアドレス + Write
	TWI_send(adrs);
	TWI_send(data >> 8);
	TWI_send(data & 0x00FF);
	TWI_stop();
}


//*******************************************************
//	AR1000 レジスタ読み込み(TWI)
//*******************************************************
ushort read_AR1000(const uchar adrs){
	ushort	data;

	TWI_start();
	TWI_send(0x20);			// スレーブアドレス + Write
	TWI_send(adrs);
	TWI_start();
	TWI_send(0x20 + 1);		// スレーブアドレス + Read
	data = TWI_recv(0);		// ACK
	data <<= 8;
	data |= TWI_recv(1);	// NACK
	TWI_stop();

	return data;
}
