//*******************************************************
//
//	FM Radio in BCL Radio Figure
//
//	Tuner Chip : RDA5802E
//
//	MCU : ATtiny13
//		hfuse:FBh	BOD 2.7V
//		lfuse:22h	0.6MHz
//
//	Compiler:	AVR-GCC		4.3.5
//				AVR-LIBC	1.7.1
//				binutils	2.20.1
//
//	2012.5.25	KAWAKAMI Yukio
//*******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>
#include	<avr/eeprom.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"main.h"
#include	"RDA5802.h"


#define	RDA5802_02H_HIGH_DEFAULT	RDA5802_DHIZ|RDA5802_DMUTE_OFF|RDA5802_MONO|RDA5802_SEEKUP
#define RDA5802_02H_LOW_DEFAULT		RDA5802_SKMODE_AROUND|RDA5802_CLK_MODE_32KHZ|RDA5802_ENABLE
#define	RDA5802_03H_LOW_DEFAULT		RDA5802_BAND_JAPAN|RDA5802_SPACE_100KHZ

PROGMEM uchar RDA5802_regs[] = {	// 1wordを High, Lowの順で格納してある
	RDA5802_02H_HIGH_DEFAULT,	// REG 02H High
	RDA5802_02H_LOW_DEFAULT,	// REG 02H Low
	0x00,						// REG 03H High
	RDA5802_03H_LOW_DEFAULT,
	RDA5802_DE_50us|RDA5802_SOFTMUTE_EN,	// REG 04H High
	0x00,		// REG 04H Low
	0x08,		// REG 05H High
	0x0A,		// REG 05H Low
	0x00,		// REG 06H High
	0x00,		// REG 06H Low
	0x46, 0xC6,
	0x50, 0x96,
	0x00, 0x00,
	0x40, 0x00, // REG 0AH
	0x00, 0x8F,
	0x00, 0x00,
	0x00, 0x00,
	0x00, 0x00,
	0x00, 0x00,
	0xF0, 0x05, // REG 10H
	0x90, 0x00,
	0xF4, 0x84,
	0x70, 0x01,
	0x40, 0xF0,
	0x21, 0x80, // REG 15H
	0x7A, 0xD0,
	0x3E, 0x40,
	0x55, 0xA9,
	0xE8, 0x48,
	0x50, 0x80, // REG 1AH 
	0x00, 0x00,
	0x37, 0xB6,
	0x40, 0x0C,
	0x07, 0x9B,
	0x4C, 0x1D,
	0x81, 0x11, // REG 20H
	0x45, 0x80,
};

void write_RDA5802(uchar buff[4]);
void read_RDA5802(uchar buff[2]);

extern volatile uchar	Timer;


//*******************************************************
//	Init FM Chip
//*******************************************************
void init_fm_chip(void){
	uchar	i;
	uchar	r;

	TWI_start();
	TWI_send(ADRS_RDA5802);			// スレーブアドレス + Write
	for (i = 0; i < sizeof(RDA5802_regs); i++){
		r = pgm_read_byte(&RDA5802_regs[i]);
		TWI_send(r);
	}
	TWI_stop();
}


//*******************************************************
//	Seek UP
//*******************************************************
ushort seek_up(void){
	uchar	data[4];
	ushort	chan;

	read_RDA5802(data);
	chan = (((ushort)(data[0] & RDA5802_READCHAN_HIGH)) << 8) | data[1];
	data[0] = RDA5802_02H_HIGH_DEFAULT|RDA5802_SEEK;
	data[1] = RDA5802_02H_LOW_DEFAULT;
	data[2] = (uchar)(chan >> 2);
	data[3] = (uchar)((chan & 3) << 6)|RDA5802_03H_LOW_DEFAULT;
	write_RDA5802(data);

	wait_stc();

	read_RDA5802(data);
	chan = (((ushort)(data[0] & RDA5802_READCHAN_HIGH)) << 8) | data[1];
	return chan + 760;
}


//*******************************************************
//	STCフラグ待ち(Timerを使って LED点滅)
//*******************************************************
void wait_stc(void){
	uchar	data[2];

	for (;;){
		wait_timer(1);
		read_RDA5802(data);
		if (data[0] & RDA5802_STC)	break;
		if (!Timer){
			Timer = 20;
			PINB |= (1<<LED);	// LED点滅
		}
	}
	PORTB |= (1<<LED);	// LED点灯
}


//*******************************************************
//	周波数設定
//*******************************************************
void set_freq(ushort freq){
	uchar	data[4];
	freq -= 760;
	data[0] = RDA5802_02H_HIGH_DEFAULT;
	data[1] = RDA5802_02H_LOW_DEFAULT;
	data[2] = (uchar)(freq >> 2);
	data[3] = (uchar)((freq & 3) << 6)|RDA5802_03H_LOW_DEFAULT|RDA5802_TUNE;
	write_RDA5802(data);
	wait_stc();
}


//*******************************************************
//	RDA5802 レジスタ書き込み
//*******************************************************
void write_RDA5802(uchar buff[4]){

	TWI_start();
	TWI_send(ADRS_RDA5802 + 0);			// スレーブアドレス + Write
	TWI_send(buff[0]);
	TWI_send(buff[1]);
	TWI_send(buff[2]);	// ch
	TWI_send(buff[3]);	// band
	TWI_stop();
}


//*******************************************************
//	RDA5802 レジスタ読み込み
//	メモリ節約のため、status と chのみ
//*******************************************************
void read_RDA5802(uchar buff[2]){

	TWI_start();
	TWI_send(ADRS_RDA5802 + 1);			// スレーブアドレス + Read
	buff[0] = TWI_recv(0);	// ACK
	buff[1] = TWI_recv(1);	// NACK
	//rssi = TWI_recv(1);	// NACK
	TWI_stop();
}
