//*******************************************************
//
//	FM Radio in BCL Radio Figure
//
//	Tuner Chip : TEA5767
//
//	注意：このソースではまだまともに動作していない。
//
//	MCU : ATtiny13
//		hfuse:FBh	BOD 2.7V
//		lfuse:22h	0.6MHz
//
//	Compiler:	AVR-GCC		4.3.5
//				AVR-LIBC	1.7.1
//				binutils	2.20.1
//
//	2012.5.25	KAWAKAMI Yukio
//*******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>
#include	<avr/eeprom.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"main.h"
#include	"TEA5767.h"

#define	DATA1_DEFAULT	0
#define	DATA2_DEFAULT	0
#define	DATA3_DEFAULT	DATA3_MS|DATA3_HLSI	// Mono, Hight Side Injection
#define	DATA4_DEFAULT	DATA4_BL|DATA4_XTAL|DATA4_HCC|DATA4_SNC	// Band Japanese, 32kHz Xtal
#define	DATA5_DEFAULT	0					// 50us

void write_TEA5767(uchar buff[5]);
void read_TEA5767(uchar buff[2]);
void set_default(uchar buff[5]);

extern volatile uchar	Timer;


//*******************************************************
//	Init FM Chip
//*******************************************************
void init_fm_chip(void){
	// TEA5767では不要
}


//*******************************************************
//	Seek UP
//*******************************************************
ushort seek_up(void){
	uchar	data[5];
	ushort	pll;

	read_TEA5767(data);			// 読み出し
	pll = (((ushort)(data[0] & DATA1_PLL)) << 8) | data[1];
	if (pll >= (PLL_91MHZ - 10)){
		pll = PLL_76MHZ;
	} else {
		pll += 10;
	}
	set_freq(pll);
	data[0] = (pll >> 8)|DATA1_SM|DATA1_MUTE;				// Seek
	data[1] = (uchar)pll;
	data[2] = DATA3_DEFAULT|DATA3_SUD;	// Seek Up
	data[3] = DATA4_DEFAULT;
	data[4] = DATA5_DEFAULT;
	write_TEA5767(data);
	wait_stc();
	read_TEA5767(data);			// 読み出し
	pll = (((ushort)(data[0] & DATA1_PLL)) << 8) | data[1];
	data[0] &= DATA1_PLL;	// Mute解除
	write_TEA5767(data);
	return pll;
}


//*******************************************************
//	STCフラグ待ち(Timerを使って LED点滅)
//*******************************************************
void wait_stc(void){
	uchar	data[5];

	for (;;){
		wait_timer(1);
		read_TEA5767(data);
		if (data[0] & DATA1_RF)	break;	// Ready Flag
		if (!Timer){
			Timer = 20;
			PINB |= (1<<LED);	// LED点滅
		}
	}
	PORTB |= (1<<LED);	// LED点灯
}


//*******************************************************
//	周波数設定
//	TEA5767では周波数ではなく、PLL値
//*******************************************************
void set_freq(ushort freq){
	uchar	data[5];

	data[0] = DATA1_DEFAULT | (freq >> 8);
	data[1] = (uchar)freq;
	data[2] = DATA3_DEFAULT;
	data[3] = DATA4_DEFAULT;
	data[4] = DATA5_DEFAULT;
	write_TEA5767(data);
	wait_stc();
}


//*******************************************************
//	TEA5767	レジスタ書き込み
//*******************************************************
void write_TEA5767(uchar buff[5]){
	uchar	i;

	TWI_start();
	TWI_send(0xC0);		// スレーブアドレス + Write
	for (i = 0; i < 5; i++){
		TWI_send(buff[i]);
	}
	TWI_stop();
}


//*******************************************************
//	TEA5767 レジスタ読み込み
//	メモリ節約の為、最初の DATA1, DATA2のみ
//*******************************************************
void read_TEA5767(uchar buff[2]){

	TWI_start();
	TWI_send(0xC1);		// スレーブアドレス + Read
	buff[0] = TWI_recv(0);	// ACK
	buff[1] = TWI_recv(1);		// NACK
	TWI_stop();
}

