//*******************************************************
//
//	FM Radio in BCL Radio Figure
//
//	MCU : ATtiny13
//		hfuse:FBh	BOD 2.7V
//		lfuse:22h	0.6MHz
//
//	Compiler:	AVR-GCC		4.3.5
//				AVR-LIBC	1.7.1
//				binutils	2.20.1
//
//	2012.5.25	KAWAKAMI Yukio
//*******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>
#include	<avr/eeprom.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"main.h"

volatile uchar	Timer;
volatile uchar	WTimer;
volatile uchar	SwCount;
volatile uchar	SwFlag;

//*******************************************************
//	タイマー0比較一致A割り込み
//*******************************************************
ISR(TIM0_COMPA_vect){

	if (Timer)	--Timer;
	if (WTimer)	--WTimer;

	if (!(PINB & (1<<SW))){
		if (SwCount < 255)	++SwCount;
	} else if (SwCount > 0){
		SwFlag = SwCount;
		SwCount = 0;
	}
}


//*******************************************************
//	メインルーチン
//*******************************************************
int main(void){
	ushort	freq;

	//cli();

	//CLKPR = (1<<CLKPCE);				// クロックプリスケーラ変更
	//CLKPR = (1<<CLKPS1)|(1<<CLKPS0);	// 1/8 4.8MHz / 8 = 600kHz

	PORTB = (1<<SW);
	DDRB = (1<<POWER)|(1<<LED);

	MCUCR = (1<<SE);		// スリープ許可、アイドリングモード

	TCCR0A = (1<<WGM01);				// CTC
	TCCR0B = (1<<CS01)|(1<<CS00);		// プリスケーラ 1/64
	OCR0A = ((F_CPU/64 + 50)/100 - 1);	// 100Hz
	TIMSK0 = (1<<OCIE0A);				// 比較一致A割り込み許可

	ACSR |= (1<<ACD);	// アナログコンパレータ禁止
	
	sei();

	// スイッチが放されるまで待つ
	while(!(PINB & (1<<SW))){
		WDR;
		SLEEP;
		if (SwCount >= 200){		// 2秒
			PORTB |= (1<<POWER);	// 電源保持
			PORTB |= (1<<LED);
		}
	}
	wait_timer(10);

	// TWI リセット
	TWI_start();
	TWI_send(0xFF);
	TWI_start();
	TWI_stop();

	init_fm_chip();

	freq = eeprom_read_word(EEPROM_FREQ);

	if (freq == 0xFFFF)	freq = 760;		// 76.0MHz
	set_freq(freq);

	SwFlag = 0;
	for(;;){
		WDR;
		SLEEP;

		if (SwCount >= 200){		// 2秒以上
			ushort	last_freq;
			
			// 電源OFF
			last_freq = eeprom_read_word(EEPROM_FREQ);
			if (last_freq != freq){
				eeprom_write_word(EEPROM_FREQ, freq);
			}
			PORTB &= ~(1<<LED);
			PORTB &= ~(1<<POWER);
		}

		if (SwFlag >= 200){			// 2秒以上
			;
		} else if(SwFlag > 0){
			if (SwFlag > 10){	// 0.1秒以上
				freq = seek_up();			// Seek Up
			}
			SwFlag = 0;
		}
	}
}


//*******************************************************
//	10ms単位時間待ち
//*******************************************************
void wait_timer(uchar tm){

	WTimer = tm;

	while(WTimer){
		WDR;
		SLEEP;
	}
}


//************************************************************
//	TWI 開始条件
//************************************************************
void TWI_start(void){

	SDA_H;
	SCL_H;
	SDA_L;
}


//************************************************************
//	TWI 8bit 送信
//************************************************************
void TWI_send(uchar data){
	char	bit;

	SCL_L;
	for (bit = 0; bit < 8; bit++){
		if (data & 0x80){
			SDA_H;
		} else {
			SDA_L;
		}
		SCL_H;
		WAIT_SCL_H;		// SCL=Hになるまで待つ
		data <<= 1;
		SCL_L;
	}

	SDA_H;
	SCL_L;

	// ACK受信 内容は無視
	SCL_H;
	SCL_L;
}


//************************************************************
//	TWI 8bit 受信
//************************************************************
uchar TWI_recv(uchar nack){
	char	bit;
	uchar	data;

	SDA_H;
	SCL_L;

	data = 0;
	for (bit = 0; bit < 8; bit++){
		SCL_H;
		WAIT_SCL_H;		// SCL=Hになるまで待つ
		data <<= 1;
		if (PINB & (1<<SDA)){
			data |= 1;
		}
		SCL_L;
	}

	// ACK/NACK 送信
	if (nack){
		SCL_L;
		SCL_H;
		SDA_H;
	} else {
		SDA_L;
		SCL_L;
		SCL_H;
	}

	SCL_L;
	SDA_H;

	return data;
}


//************************************************************
//	TWI 停止条件
//************************************************************
void TWI_stop(void){

	SDA_L;
	SCL_H;
	SDA_H;
}

